package org.bitbucket.bradleysmithllc.etlunit;

import org.bitbucket.bradleysmithllc.etlunit.ClassDirector;
import org.bitbucket.bradleysmithllc.etlunit.ClassResponder;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestClass;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestMethod;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestOperation;

public class ConditionalDirector implements ClassDirector
{
	public enum condition
	{
		and,
		or
	}

	private final ClassDirector ldirector;
	private final condition icondition;
	private final ClassDirector rdirector;

	public ConditionalDirector(ClassDirector ldirector, condition icondition, ClassDirector rdirector) {
		this.ldirector = ldirector;
		this.icondition = icondition;
		this.rdirector = rdirector;
	}

	@Override
	public void beginBroadcast() {
		try
		{
			ldirector.beginBroadcast();
		}
		finally
		{
			rdirector.beginBroadcast();
		}
	}

	@Override
	public response_code accept(ETLTestClass cl) {
		response_code lresponse = ldirector.accept(cl);
		response_code rresponse = rdirector.accept(cl);

		return check(lresponse, rresponse);
	}

	private response_code check(response_code lresponse, response_code rresponse) {
		switch (icondition) {
			case and:
				if (lresponse == response_code.reject || rresponse == response_code.reject)
				{
					return response_code.reject;
				}
				else if (lresponse == response_code.defer || rresponse == response_code.defer)
				{
					return response_code.defer;
				}

				return response_code.accept;
			case or:
				if (lresponse == response_code.reject && rresponse == response_code.reject)
				{
					return response_code.reject;
				}
				else if (lresponse == response_code.defer && rresponse == response_code.defer)
				{
					return response_code.defer;
				}

				return response_code.accept;
			default:
				throw new IllegalStateException();
		}
	}

	@Override
	public response_code accept(ETLTestMethod mt) {
		response_code lresponse = ldirector.accept(mt);
		response_code rresponse = rdirector.accept(mt);

		return check(lresponse, rresponse);
	}

	@Override
	public response_code accept(ETLTestOperation op) {
		response_code lresponse = ldirector.accept(op);
		response_code rresponse = rdirector.accept(op);

		return check(lresponse, rresponse);
	}

	@Override
	public void endBroadcast() {
		try
		{
			ldirector.endBroadcast();
		}
		finally
		{
			rdirector.endBroadcast();
		}
	}

	public ClassDirector getLdirector() {
		return ldirector;
	}

	public condition getCondition() {
		return icondition;
	}

	public ClassDirector getRdirector() {
		return rdirector;
	}
}