package org.bitbucket.bradleysmithllc.etlunit.feature.debug;

import org.bitbucket.bradleysmithllc.etlunit.*;
import org.bitbucket.bradleysmithllc.etlunit.context.VariableContext;
import org.bitbucket.bradleysmithllc.etlunit.feature.AbstractFeature;
import org.bitbucket.bradleysmithllc.etlunit.feature.FeatureModule;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestOperation;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestValueObject;

@FeatureModule
public class DebugFeatureModule extends AbstractFeature
{
	@Override
	public String getFeatureName()
	{
		return "debug";
	}

	@Override
	public ClassListener getListener()
	{
		return new NullClassListener()
		{
			@Override
			public action_code process(ETLTestOperation op, ETLTestValueObject obj, final VariableContext context, ExecutionContext econtext)
					throws TestAssertionFailure, TestExecutionError, TestWarning
			{
				String name = op.getOperationName();

				if (name.equals("error"))
				{
					ETLTestValueObject errorId = obj.query("error-id");

					if (errorId != null)
					{
						throw new TestExecutionError(obj.getValueAsMap().get("message").getValueAsString(),
								errorId.getValueAsString());
					}
					else
					{
						throw new TestExecutionError(obj.getValueAsMap().get("message").getValueAsString());
					}
				}
				else if (name.equals("die"))
				{
					throw new RuntimeException("Blech");
				}
				else if (name.equals("fail"))
				{
					ETLTestValueObject errorId = obj.query("failure-id");

					if (errorId != null)
					{
						throw new TestAssertionFailure(op.getOperands().getValueAsMap().get("message").getValueAsString(),
								errorId.getValueAsString());
					}
					else
					{
						throw new TestAssertionFailure(op.getOperands().getValueAsMap().get("message").getValueAsString());
					}
				}
				else if (name.equals("warn"))
				{
					ETLTestValueObject errorId = obj.query("warning-id");

					if (errorId != null)
					{
						throw new TestWarning(op.getOperands().getValueAsMap().get("message").getValueAsString(),
								errorId.getValueAsString());
					}
					else
					{
						throw new TestWarning(op.getOperands().getValueAsMap().get("message").getValueAsString());
					}
				}
				else if (name.equals("assert"))
				{
					ETLTestValueObject var = obj.query("variable");
					ETLTestValueObject value = obj.query("value");

					if (var != null && value != null)
					{
						String varName = var.getValueAsString();

						ETLTestValueObject cvalue = context.getValue(varName);

						if (!cvalue.getJsonNode().equals(value.getJsonNode()))
						{
							throw new TestAssertionFailure("Variable " + varName + " does not have the correct value");
						}

						return action_code.handled;
					}
				}

				return action_code.defer;
			}
		};
	}

	@Override
	public boolean requiredForSimulation() {
		return true;
	}
}