package org.bitbucket.bradleysmithllc.etlunit.io.file;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import org.bitbucket.bradleysmithllc.json.validator.ClasspathSchemaResolver;
import org.bitbucket.bradleysmithllc.json.validator.JsonSchemaValidationException;
import org.bitbucket.bradleysmithllc.json.validator.JsonUtils;
import org.bitbucket.bradleysmithllc.json.validator.JsonValidator;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.regex.Pattern;

class FlatFileType implements DataFormatValidator
{
	private final String id;
	private final Pattern pattern;

	public FlatFileType(String id, JsonNode node)
	{
		this.id = id;
		this.pattern = Pattern.compile(node.asText());
	}

	public String getId()
	{
		return id;
	}

	public Pattern getPattern()
	{
		return pattern;
	}

	public static Map<String, FlatFileType> load(String types)
	{
		Map<String, FlatFileType> typeMap = new HashMap<String, FlatFileType>();

		try
		{
			JsonValidator
					vlad =
					new JsonValidator("org/bitbucket/bradleysmithllc/etlunit/io/file/ffml/ffmlTypes.jsonSchema",
							new ClasspathSchemaResolver(FlatFileType.class));

			JsonNode jsonNode = JsonUtils.loadJson(types);
			vlad.validate(jsonNode);

			ObjectNode on = (ObjectNode) jsonNode;

			Iterator<Map.Entry<String, JsonNode>> fields = on.fields();

			while (fields.hasNext())
			{
				Map.Entry<String, JsonNode> field = fields.next();

				typeMap.put(field.getKey(), new FlatFileType(field.getKey(), field.getValue()));
			}

			return typeMap;
		}
		catch (JsonSchemaValidationException e)
		{
			throw new IllegalArgumentException(e);
		}
	}
}